import warnings


def dataset_warning(dataset):
    warnings.warn(
        "This library provides access to data sets without claiming ownership over them or defining their licensing terms. "
        "Users who download data are responsible for checking the license of each individual data set."
    )
